@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import typing
 
@@ -11,10 +13,85 @@ from . import utils
 from .newton_solver import MechanicsNewtonSolver
 from .newton_solver import MechanicsNewtonSolver_ODE
 
+if typing.TYPE_CHECKING:
+    from .models import em_model
 
 logger = utils.getLogger(__name__)
 
 
+def setup_solver(
+    coupling: em_model.BaseEMCoupling,
+    ActiveModel,
+    bnd_rigid: bool = config.Config.bnd_rigid,
+    pre_stretch: typing.Optional[typing.Union[dolfin.Constant, float]] = None,
+    traction: typing.Union[dolfin.Constant, float] = None,
+    spring: typing.Union[dolfin.Constant, float] = None,
+    fix_right_plane: bool = config.Config.fix_right_plane,
+    debug_mode: bool = config.Config.debug_mode,
+    set_material: str = "",
+    linear_solver="mumps",
+    use_custom_newton_solver: bool = config.Config.mechanics_use_custom_newton_solver,
+    state_prev=None,
+):
+    """Setup mechanics model with dirichlet boundary conditions or rigid motion."""
+
+    if ActiveModel is None:
+        return None
+    logger.info("Set up mechanics model")
+
+    # Use parameters from Biaxial test in Holzapfel 2019 (Table 1).
+    material_parameters = dict(
+        a=2.28,
+        a_f=1.686,
+        b=9.726,
+        b_f=15.779,
+        a_s=0.0,
+        b_s=0.0,
+        a_fs=0.0,
+        b_fs=0.0,
+    )
+
+    active_model = ActiveModel(coupling=coupling, parameters=coupling.cell_params())
+    material = pulse.HolzapfelOgden(
+        active_model=active_model,
+        parameters=material_parameters,
+    )
+
+    if set_material == "Guccione":
+        material_parameters = pulse.Guccione.default_parameters()
+        material_parameters["CC"] = 2.0
+        material_parameters["bf"] = 8.0
+        material_parameters["bfs"] = 4.0
+        material_parameters["bt"] = 2.0
+
+        material = pulse.Guccione(
+            active_model=active_model,
+            parameters=material_parameters,
+        )
+
+    problem = create_problem(
+        material=material,
+        geo=coupling.geometry,
+        bnd_rigid=bnd_rigid,
+        pre_stretch=pre_stretch,
+        traction=traction,
+        spring=spring,
+        fix_right_plane=fix_right_plane,
+        linear_solver=linear_solver,
+        use_custom_newton_solver=use_custom_newton_solver,
+        debug_mode=debug_mode,
+    )
+
+    if state_prev is not None:
+        problem.state.assign(state_prev)
+
+    problem.solve()
+    coupling.register_mech_model(problem)
+    coupling.print_mechanics_info()
+
+    return problem
+
+
 class ContinuationBasedMechanicsProblem(pulse.MechanicsProblem):
     def __init__(self, *args, **kwargs):
         self._use_custom_newton_solver = kwargs.pop("use_custom_newton_solver", False)
@@ -83,6 +160,7 @@ class MechanicsProblem(ContinuationBasedMechanicsProblem):
 
     def _init_forms(self, init_solver: bool = True):
         u, p = dolfin.split(self.state)
+        v, q = dolfin.split(self.state_test)
 
         # Some mechanical quantities
         self._F = dolfin.variable(pulse.DeformationGradient(u))
@@ -99,6 +177,10 @@ class MechanicsProblem(ContinuationBasedMechanicsProblem):
             self.state_test,
         )
 
+        external_work = self._external_work(u, v)
+        if external_work is not None:
+            self._virtual_work += external_work
+
         self._set_dirichlet_bc()
         self._jacobian = dolfin.derivative(
             self._virtual_work,
@@ -123,7 +205,7 @@ class MechanicsProblem(ContinuationBasedMechanicsProblem):
         self.solver = cls(
             problem=self._problem,
             state=self.state,
-            update_cb=self.material.active.update_prev,
+            # update_cb=self.material.active.update_prev,
             parameters=self.solver_parameters,
         )
 
@@ -133,9 +215,6 @@ class MechanicsProblem(ContinuationBasedMechanicsProblem):
         getattr(self.solver, "check_overloads_called", None)
         return newton_iteration, newton_converged
 
-    def update_lmbda_prev(self):
-        self.lmbda_prev.vector()[:] = self.lmbda.vector()
-
 
 class RigidMotionProblem(MechanicsProblem):
     def _init_spaces(self):
@@ -181,6 +260,10 @@ class RigidMotionProblem(MechanicsProblem):
             self.state_test,
         )
 
+        external_work = self._external_work(u, v)
+        if external_work is not None:
+            self._virtual_work += external_work
+
         self._virtual_work += dolfin.derivative(
             RigidMotionProblem.rigid_motion_term(
                 mesh=self.geometry.mesh,
